<?php

namespace EventPortal\Domain\Table\Factories;

use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Table\DataObjects\Column;
use EventPortal\Domain\Table\DataObjects\Filter;
use EventPortal\Domain\Table\DataObjects\LabelValue;
use EventPortal\Domain\Table\GroupedActions;
use EventPortal\Domain\Table\Table;
use EventPortal\Domain\Util\DefaultConfig;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\MorphMany;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;

class TableFactory
{
   /** @var HasFields */
    protected $model;

    /** @var Collection */
    protected $allAvailableColumns;

    /** @var Collection */
    protected $columnsToDisplay;

    /** @var Collection */
    protected $availableFilters;

    /** @var Collection */
    protected $filters;

    protected $order;

    protected $sort;

    /** @var int */
    protected $page = 1;

    /** @var int */
    protected $size = 5;

    /** @var string */
    protected $query;

    /** @var \EventPortal\Domain\Table\GroupedActions */
    private $globalActions;

    public function make(string $modelClass): self
    {
        $this->model = new $modelClass;

        $this->filters = collect();
        $this->allAvailableColumns = collect();
        $this->columnsToDisplay = collect();

        $allAvailableColumns = $this->getColumns();
        $defaultColumns = $this->defaultColumns($allAvailableColumns);

        $this->allAvailableColumns = $this->allAvailableColumns($allAvailableColumns);
        $this->availableFilters = $this->availableFilters($defaultColumns);
        $this->columnsToDisplay = $this->columnsToDisplay($defaultColumns);

        return $this;
    }

    public function columns(?string $selectedColumns): self
    {
        if ($selectedColumns !== null) {
            $columns = Str::of($selectedColumns)->explode(',')->toArray();

            $columns = $this->getColumns($columns);

            $this->columnsToDisplay = $this->columnsToDisplay($columns);
            $this->availableFilters = $this->availableFilters($columns);
        }

        return $this;
    }

    public function filters(?array $filters): self
    {
        if ($filters !== null) {
            $this->filters = collect($filters)->filter();
        }

        return $this;
    }

    public function page(?string $page): self
    {
        $this->page = $page;

        return $this;
    }

    public function size(?string $size): self
    {
        if ($size !== null) {
            $this->size = $size;
        }

        return $this;
    }

    public function search(?string $query): self
    {
        $this->query = $query;

        return $this;
    }

    public function order(?string $order = 'asc'): self
    {
        $this->order = $order;

        return $this;
    }

    public function sort(?string $sort): self
    {
        $this->sort = $sort;

        return $this;
    }

    public function globalActions(GroupedActions $actions): self
    {
        $this->globalActions = $actions;

        return $this;
    }

    public function get(): Table
    {
        return Table::make()
            ->currentFilters($this->filters)
            ->currentQuery($this->query)
            ->currentOrder($this->order)
            ->currentSort($this->sort)
            ->defaultPageSize($this->size)
            ->withColumns($this->columnsToDisplay)
            ->withAllColumns($this->allAvailableColumns)
            ->withGlobalActions($this->globalActions)
            ->withFiltersFields($this->availableFilters)
            ->withData($this->getData());
    }

    private function getData(): LengthAwarePaginator
    {
        return $this
            ->model
            ->with(['fields' => function (MorphMany $query): MorphMany {
                return $query
                    ->with('response')
                    ->forModelType($this->model)
                    ->onlyChildren()
                    ->whereIn('name', Arr::pluck($this->columnsToDisplay, 'key'));
            }])
            ->when($this->filters, function ($query) {
                foreach ($this->filters as $filter => $value) {
                    $query
                        ->whereHas('fields', static function ($query) use ($filter, $value) {
                            return $query
                                ->where('name', '=', $filter)
                                ->hasResponseOf($value);
                        });
                }

                return $query;
            })
            ->when($this->query, function ($query) {
                return $query
                    ->whereHas('fields', function ($query) {
                        return $query
                            ->whereIn('name', Arr::pluck($this->columnsToDisplay, 'key'))
                            ->hasResponseLike($this->query);
                    });
            })
            ->when($this->sort, function ($query) {
                return $query
                    ->whereHas('fields.response', function ($query) {
                        if ($this->order === 'desc') {
                            return $query->orderByDesc('responses.data->value');//Think this works for single, need to get it working for double
                        }

                        return $query->addSelect(['responses.data->value' => Field::select('')]);
//                        return $query->orderBy('responses.data->value');//Think this works for single, need to get it working for double
                        //TODO need to get it working sorta like $query->orderBy('responses.data->value->value');
                    });
            })
            ->paginate($this->size);
    }

    private function getColumns(?array $columns = []): Collection
    {
        return Field::query()
            ->select(['name', 'label', 'type', 'data'])
            ->distinct()
            ->forModelType($this->model)
            ->onlyChildren()
            ->when($columns !== [], function (Builder $query) use ($columns): Builder {
                return $query->whereIn('name', $columns);
            })
            ->get()
            ->unique('name');
    }

    private function availableFilters(Collection $columns): Collection
    {
        return $columns
            ->whereIn('type', [FieldType::Select(), FieldType::MultiSelect()])
            ->mapInto(Filter::class)
            ->values();
    }

    private function allAvailableColumns(Collection $columns): Collection
    {
        return $columns
            ->mapInto(LabelValue::class)
            ->values();
    }

    private function columnsToDisplay(Collection $columns): Collection
    {
        return $columns
            ->mapInto(Column::class)
            ->values();
    }

    private function defaultColumns(Collection $columns): Collection
    {
        $defaultColumns = DefaultConfig::for(get_class($this->model))->tableColumns();

        return $columns->whereIn('name', $defaultColumns);
    }
}
