<?php

namespace EventPortal\Tests\Unit\Models;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class FieldTest extends TestCase
{
    use DatabaseMigrations;

    /** @test **/
    public function can_scope_for_model()
    {
        $eventA = factory(Event::class)->create();
        $fieldA = factory(Field::class)->create([
            'model_id' => $eventA->id,
            'model_type' => Event::class,
        ]);

        $eventB = factory(Event::class)->create();
        $fieldB = factory(Field::class)->create([
            'model_id' => $eventB->id,
            'model_type' => Event::class,
        ]);

        $fields = Field::forModel($eventA)->get();

        $this->assertCount(1, $fields);
        $this->assertContainsModel($fieldA, $fields);
        $this->assertNotContainsModel($fieldB, $fields);
    }

    /** @test **/
    public function can_scope_to_only_sections()
    {
        $fieldA = factory(Field::class)->create(['section_id' => 1]);
        $fieldB = factory(Field::class)->create();

        $fields = Field::onlySections()->get();

        $this->assertCount(1, $fields);
        $this->assertContainsModel($fieldB, $fields);
        $this->assertNotContainsModel($fieldA, $fields);
    }

    /** @test **/
    public function can_scope_to_only_children()
    {
        $fieldA = factory(Field::class)->create(['section_id' => 1]);
        $fieldB = factory(Field::class)->create();

        $fields = Field::onlyChildren()->get();

        $this->assertCount(1, $fields);
        $this->assertContainsModel($fieldA, $fields);
        $this->assertNotContainsModel($fieldB, $fields);
    }

    /** @test **/
    public function can_determine_if_it_is_a_child()
    {
        $fieldA = factory(Field::class)->create(['section_id' => 1]);
        $fieldB = factory(Field::class)->create(['section_id' => null]);

        $this->assertTrue($fieldA->isChild());
        $this->assertFalse($fieldB->isChild());
    }

    /** @test **/
    public function can_determine_if_it_is_a_parent()
    {
        $fieldA = factory(Field::class)->create(['section_id' => 1]);
        $fieldB = factory(Field::class)->create(['section_id' => null]);

        $this->assertFalse($fieldA->isParent());
        $this->assertTrue($fieldB->isParent());
    }

    /** @test **/
    public function can_determine_if_has_response_with_value()
    {
        $this->markTestSkipped('Not working with SQLite');

        $fieldA = factory(Field::class)->create(['section_id' => 1]);
        $fieldB = factory(Field::class)->create(['section_id' => 1]);

        $responseA = factory(Response::class)->create([
            'field_id' => $fieldA->id,
            'value' => [
                ['label' => 'Hello World', 'value' => 'hello-world'],
                ['label' => 'Testing', 'value' => 'testing']
            ],
        ]);

        $responseB = factory(Response::class)->create([
            'field_id' => $fieldB->id,
            'value' => 'Goodbye World',
        ]);

        $actual = Field::hasResponseOf('hello-world')->first();
        $this->assertSameModel($fieldA, $actual);

        $actual = Field::hasResponseOf('goodbye-world')->first();
        $this->assertSameModel($fieldB, $actual);
    }
}
