<?php

namespace EventPortal\Tests\Unit\Table;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Collection;

class TableTest extends TestCase
{
    use DatabaseMigrations;

    /** @test **/
    public function can_make()
    {
        $table = Table::make(Event::class)->get();

        $this->assertInstanceOf(\EventPortal\Domain\Table\Table::class, $table);
    }

    /** @test **/
    public function uses_model_column_defaults_if_no_columns_specified()
    {
        factory(Event::class)->state('default-data')->create();

        $table = Table::make(Event::class)->get();

        $this->assertInstanceOf(Collection::class, $table->columns);
        $this->assertCount(5, $table->columns);
    }

    /** @test **/
    public function can_specify_columns_to_use()
    {
        factory(Event::class)->state('default-data')->create();

        $table = Table::make(Event::class)->columns('name,status')->get();

        $this->assertInstanceOf(Collection::class, $table->columns);
        $this->assertCount(3, $table->columns);
    }

    /** @test **/
    public function can_handle_specifing_columns_that_do_not_exist()
    {
        factory(Event::class)->state('default-data')->create();

        $table = Table::make(Event::class)->columns('name,status,url')->get();

        $this->assertInstanceOf(Collection::class, $table->columns);
        $this->assertCount(3, $table->columns);
    }

    /** @test **/
    public function filters()
    {
        $event = factory(Event::class)->create();

        $field = factory(Field::class)
            ->states('Select')
            ->create([
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => 1,
                'label' => 'Event Status',
                'name' => 'status',
                'data' => [
                    'options' => [
                        [
                            'label' => 'Draft',
                            'value' => 'draft',
                        ],
                        [
                            'label' => 'Published',
                            'value' => 'published',
                        ]
                    ]
                ]
            ]);

        $table = Table::make(Event::class)->columns('status')->get();

        $this->assertInstanceOf(Collection::class, $table->filterFields);
        $this->assertCount(1, $table->filterFields);

    }

    /** @test */
    public function order()
    {
        $eventB = factory(Event::class)->create();
        $sectionB = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $eventB->id,
                'model_type' => get_class($eventB),
            ]);

        $fieldB = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $eventB->id,
                'model_type' => get_class($eventB),
                'section_id' => $sectionB->id,
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldB->id,
                'value' => 'B',
            ]);

        $eventA = factory(Event::class)->create();
        $sectionA = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $eventA->id,
                'model_type' => get_class($eventA),
            ]);

        $fieldA = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $eventA->id,
                'model_type' => get_class($eventA),
                'section_id' => $sectionA->id,
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldA->id,
                'value' => 'A',
            ]);

//        ->order('desc')
        $table = Table::make(Event::class)->columns('name')->sort('name')->get();
        dd($table->data);

        $this->assertInstanceOf(Collection::class, $table->filterFields);
        $this->assertCount(1, $table->filterFields);
    }
}
